import type { PlasmoMessaging } from '@plasmohq/messaging';

import { API_URL, DEFAULT_DOMAIN_CONFIG } from '~utils/constants';
import { noop } from '~utils/error';
import { storage } from '~utils/storage';
import type { DomainConfig } from '~utils/types';

import updateIconHandler from '../extension/updateIcon';

const handler: PlasmoMessaging.MessageHandler<Request, Response> = async (req, res) => {
  const { domain } = req.body ?? {};
  const { tab } = req.sender || {};

  if (domain) {
    let data = (await storage.get<DomainConfig>(domain)) || DEFAULT_DOMAIN_CONFIG;
    const now = Date.now();

    if ((data.issue?.expiresAt && now > data.issue.expiresAt) || !data.issue?.expiresAt) {
      const response = await fetch(`${API_URL}/issues/${domain}/`);

      if (response.status === 429) {
        res.send({ data, success: true });
        return;
      }

      const issue = await response.json();

      if (issue.success) {
        data = { ...data, issue: { ...issue.data, expiresAt: now + 8 * 60 * 60 * 1000 } };
        await storage.set(domain, data);

        if (tab?.id !== undefined) {
          await updateIconHandler(
            { body: { domain }, name: 'extension/updateIcon', sender: { tab } },
            { send: noop }
          );
        }

        res.send({ data, success: true });
        return;
      }

      data = { ...data, issue: { expiresAt: now + 24 * 60 * 60 * 1000 } };
      await storage.set(domain, data);
    }

    res.send({ data, success: true });
    return;
  }

  res.send({ success: false });
};

interface Request {
  readonly domain: string;
}

interface Response {
  readonly data?: DomainConfig;
  readonly success: boolean;
}

export default handler;
