import type { PlasmoMessaging } from '@plasmohq/messaging';

import { API_URL } from '~utils/constants';
import { storage } from '~utils/storage';

const handler: PlasmoMessaging.MessageHandler<never, Response> = async (req, res) => {
  try {
    const response = await fetch(`${API_URL}/version/`);

    if (response.status === 429) {
      res.send({ success: false });
      return;
    }

    const { data } = await response.json();
    const { version } = chrome.runtime.getManifest();

    if (data !== version) {
      await storage.set('updateAvailable', data);
      res.send({ success: true });
      return;
    }

    await storage.remove('updateAvailable');
    res.send({ success: false });
  } catch {
    res.send({ success: false });
  }
};

interface Response {
  readonly success: boolean;
}

export default handler;
