import type { PlasmoMessaging } from '@plasmohq/messaging';
import offIcon from 'url:~assets/off.png';
import onIcon from 'url:~assets/on.png';
import warnIcon from 'url:~assets/warn.png';

import { DEFAULT_DOMAIN_CONFIG, DEFAULT_EXTENSION_DATA } from '~utils/constants';
import { validateSupport } from '~utils/domain';
import { storage } from '~utils/storage';
import type { DomainConfig, ExtensionData } from '~utils/types';

const handler: PlasmoMessaging.MessageHandler<Request, Response> = async (req, res) => {
  const { domain } = req.body || {};
  const { frameId, tab } = req.sender || {};

  if (domain && frameId === 0 && tab?.id !== undefined) {
    const config = (await storage.get<DomainConfig>(domain)) || DEFAULT_DOMAIN_CONFIG;
    const data = (await storage.get<ExtensionData>('data')) || DEFAULT_EXTENSION_DATA;

    if (
      config.on &&
      tab.url &&
      validateSupport(new URL(tab.url).hostname, data.exclusions.domains)
    ) {
      if (config.issue?.url) {
        await chrome.action.setIcon({ path: warnIcon, tabId: tab.id });
        res.send({ success: true });
        return;
      }

      await chrome.action.setIcon({ path: onIcon, tabId: tab.id });
      res.send({ success: true });
    } else {
      await chrome.action.setIcon({ path: offIcon, tabId: tab.id });
      res.send({ success: true });
    }
  }

  res.send({ success: false });
};

interface Request {
  readonly domain: string;
}

interface Response {
  readonly success: boolean;
}

export default handler;
