import { sendToBackground } from '@plasmohq/messaging';
import { useStorage } from '@plasmohq/storage/hook';
import { useCallback, useEffect, useRef, useState } from 'react';

import { storage } from '~utils/storage';
import type { ExtensionData } from '~utils/types';

export function useData(): UseDataResult {
  const timeoutId = useRef<number | undefined>(undefined);
  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [isRefetched, setIsRefetched] = useState<boolean>(false);
  const [data] = useStorage<ExtensionData>({ instance: storage, key: 'data' });

  const refetch = useCallback(async () => {
    setIsLoading(true);
    await sendToBackground({ name: 'database/refresh' });
    setIsLoading(false);
    setIsRefetched(true);
    timeoutId.current = window.setTimeout(() => setIsRefetched(false), 60000);
  }, []);

  useEffect(() => {
    return () => {
      window.clearTimeout(timeoutId.current);
    };
  }, []);

  return {
    data,
    isLoading,
    isRefetched,
    refetch,
  };
}

export interface UseDataResult {
  readonly data?: ExtensionData;
  readonly isLoading: boolean;
  readonly isRefetched: boolean;
  readonly refetch: () => Promise<void>;
}
